/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.docware;

import casa.dodwan.util.BufferAccessException;
import casa.dodwan.util.Bufferizer;
import casa.dodwan.util.Compressor;

public class DescriptorZipBufferizer extends Bufferizer<Descriptor> {

    private Bufferizer<Descriptor> db_ = new DescriptorBufferizer();

    // ------------------------------------------------------------
    public Descriptor fromBuffer(byte[] buffer, int offset, int length)
	throws BufferAccessException {

	try {
	    byte[] tmp = Compressor.uncompress(buffer, offset, length);
	    return db_.fromBuffer(tmp, 0, tmp.length);
	}
	catch (Exception e) {
	    throw new BufferAccessException();
	}
    }

    // ------------------------------------------------------------
    public int toBuffer(Descriptor desc, byte[] buffer, int offset)
	throws BufferAccessException {
	
	try {
	    byte[] tmp1 = db_.toBuffer(desc);
	    byte[] tmp2 = Compressor.compress(tmp1);
	    System.arraycopy(tmp2, 0, buffer, offset, tmp2.length);
	    casa.util.Logger.log("zip", "size=" 
					+ tmp2.length
					+ ",ratio="
					+ (100 * tmp2.length / tmp1.length)
					+ "%");
	    return tmp2.length;
	}
	catch (Exception e) {
	    throw new BufferAccessException();
	}
    }

    // ------------------------------------------------------------
    public byte[] toBuffer(Descriptor desc)
	throws BufferAccessException {

	try {
	    byte[] tmp = db_.toBuffer(desc);
	    return Compressor.compress(tmp);
	}
	catch (Exception e) {
	    throw new BufferAccessException();
	}
    }

    // ============================================================
    /**
     * @undocumented
     */
    public static void main(String[] args) throws Exception {

	Descriptor d1 = new Descriptor();
	d1.setAttribute("att1", "fg");
	d1.setAttribute("att2", "rg jg");
	d1.setAttribute("att3", "pg");

	System.out.println("d1=" + d1);

	Bufferizer<Descriptor> db = new DescriptorZipBufferizer();
	db.toFile(d1, "/tmp/d1");

	Descriptor tmp = db.fromFile("/tmp/d1");
	System.out.println("d1 from file ok: " + tmp.equals(d1));
    }
}
