/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.docware;

import casa.dodwan.util.BufferAccessException;
import casa.dodwan.util.Bufferizer;

public class PayloadBufferizer extends Bufferizer<Payload> {
    
    // ------------------------------------------------------------
    public Payload fromBuffer(byte[] buffer)
	throws BufferAccessException {

	return fromBuffer(buffer, 0, buffer.length);
    }

    // ------------------------------------------------------------
    public Payload fromBuffer(byte[] buffer, int offset, int length)
	throws BufferAccessException {
	
	byte[] buf = new byte[length];
	System.arraycopy(buffer, offset, buf, 0, length);
	
	Payload payload = new Payload(buf);
	return payload;
    }
    
    // ------------------------------------------------------------
    public byte[] toBuffer(Payload payload)
	throws BufferAccessException {
	
	byte[] buf = payload.getBytes();
	byte[] result = new byte[buf.length];

	System.arraycopy(buf, 0, result, 0, buf.length);
	
	return result;
    }

    // ------------------------------------------------------------
    public int toBuffer(Payload payload, byte[] buffer, int offset)
	throws BufferAccessException {
	
	byte[] buf = payload.getBytes();
	
	System.arraycopy(buf, 0, buffer, offset, buf.length);
	
	return buf.length;
    }

}
