/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/


package casa.dodwan.hub;

public class hubd {

    // ------------------------------------------------------------
    public static void exit(int error) {
	help();
	System.exit(1);
    }
    
    // ------------------------------------------------------------
    public static void help() {
		System.out.println("Possible options:\n"
				   + " -Dport=[port number]\n"
				   + " -Dperiod=[value in seconds]\n"
				   + " -Dlatency=[value in seconds]\n"
				   + " -Ddir_mode=[true|false]");
    }
    
    // ------------------------------------------------------------
    public static void main(String[] args) throws Exception {
	
	if ((args.length > 0) && (args[0].startsWith("-h"))) {
	    help();
	    System.exit(0);
	}

	int port;
	String str = System.getProperty("port");
	if (str != null)
	    port = Integer.parseInt(str);
	else
	    port = 4500;

	long latency;
	str = System.getProperty("latency");
	if (str != null)
	    latency = Long.parseLong(str);
	else
	    latency = 60;

	long period;
	str = System.getProperty("period");
	if (str != null)
	    period = Long.parseLong(str);
	else
	    period = 10;

	str = System.getProperty("dir_mode");
	boolean dir_mode = (str != null) && str.equals("true");

	System.out.println("  UDP port      : " 
			   + port);
	if (period != 0) {
	    System.out.println("  Node purge    : " 
			       + period + " sec");
	    System.out.println("  Node latency  : " + latency + " sec");
	}
	System.out.println("  Directory mode: " 
			   + dir_mode);
	System.out.println();


	Environment simulEnv = 
	    new FullConnectivityEnvironment();
	Hub hub = new Hub(port, simulEnv, dir_mode);
	hub.latency = latency;
	hub.period  = period;

	if (period != 0)
	    hub.start();
    }
}
