/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;

import java.net.ServerSocket;

public class ConsoleServer extends Thread {

    private Console console_;
    
    private ServerSocket serverSocket_;
    
    public Verbosity verbosity = new Verbosity();
    
    // ------------------------------------------------------------
    public ConsoleServer(int port, 
			 Console console)
	throws Exception {

	setDaemon(true);
	
	serverSocket_ = new ServerSocket(port);
	console_ = console;
    }

    // ------------------------------------------------------------
    /**
     * Returns the port number served by this ConsoleServer.
     */
    public int getLocalPort() {
	if (serverSocket_ ==  null)
	    return -1;
	return serverSocket_.getLocalPort();
    }

    // ------------------------------------------------------------
    @Override
	public void run() {
	
	java.net.Socket clientSocket = null;
	
	if (verbosity.isEnabled())
	    System.out.println("ConsoleServer.run()");
	
	while (true) {
	    try {
		clientSocket = serverSocket_.accept();
		
		if (verbosity.isEnabled())
		    System.out.println("ConsoleServer.run(): accepted a new client on socket "
				       + clientSocket);
		
		ConsoleClientHandler clientHandler = 
		    new ConsoleClientHandler(clientSocket, console_);
		clientHandler.start();
	    } catch (Exception e) {
		System.err.println("ConsoleServer.run(): " + e);
	    }
	}
    }
    
  
    // ------------------------------------------------------------
       public void close() {
    	try {
    		serverSocket_.close();
    	} catch (Exception e) {
    		System.err.println("ConsoleServer.run(): " + e);
	    }
    }
       
}
