/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;

import java.util.Locale;
import java.math.BigInteger;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.TimeZone;
import java.util.Date;
import java.util.Collection;
import java.util.BitSet;
import java.util.List;
import java.util.LinkedList;
import java.util.Set;
import java.util.HashSet;
import java.net.InetAddress;
import java.net.InetSocketAddress;
import java.util.Map;

public class DataFormatting {
    
    static private SimpleDateFormat dateFormat_ = null; 
    
    // ------------------------------------------------------------
    public static String toString(InetAddress addr) {
	
	if (addr == null)
	    return "null";
	else
	    return addr.getHostAddress();
    }
    
    // ------------------------------------------------------------
    public static String toString(InetSocketAddress addr) {
	
	if (addr == null)
	    return "null";
	else
	    return toString(addr.getAddress())
		+ ":" + addr.getPort();
    }

    // ------------------------------------------------------------
    public static InetSocketAddress toInetSocketAddress(String str) {

	// The value of the string should be of the form
	// "hostname/ip_address:port" or "ip_address:port".
	// 
	// Let us extract the ip_address and port number from this
	// string, and return an InetSocketAddress based on these
	// values.
	int idx1 = str.indexOf("/");
	int idx2 = str.lastIndexOf(":");
	String ip_addr = str.substring(idx1+1, idx2);
	String str_port = str.substring(idx2+1);
	int port = Integer.parseInt(str_port);

	return new InetSocketAddress(ip_addr, port);
    }
    
    // ------------------------------------------------------------
    public static String toString(String str, int width) {
	
	if (str.length() > width)
	    return str.substring(0, width - 1);
	
	String result = str;
	for (int i=0; i < width - str.length(); i++)
	    result += " ";
	return result;
    }
    
    // ------------------------------------------------------------
    private static DateFormat dateFormat() {
	
	if (dateFormat_ == null) {
	    dateFormat_ = new SimpleDateFormat("EEE MMM dd HH:mm:ss zzz yyyy", 
					       Locale.ENGLISH);
	    dateFormat_.setTimeZone(TimeZone.getTimeZone("GMT"));
	}
	return dateFormat_;
    }
    
    // ------------------------------------------------------------
    public static synchronized Date from_date(String str) throws Exception {
	
     	Date date = dateFormat().parse(str);
     	return date;
    }
    
    // ------------------------------------------------------------
    public static synchronized String to_date(Date date) {
	
	return dateFormat().format(date);
    }
    
    // ------------------------------------------------------------
    public static long from_hh_mm_ss(String str) throws Exception {
	
	long hh, mm, ss;
	try {
	    
	    String[] hhmmss = str.split(":", 0);
	    // String[] hhmmss = StringHelper.split(str, ":");
	    if (hhmmss.length != 3)
		throw new Exception("Failed to parse hh:mm:ss pattern");
	    
	    hh = Long.parseLong(hhmmss[0]);
	    mm = Long.parseLong(hhmmss[1]);
	    ss = Long.parseLong(hhmmss[2]);
	}
	catch (Exception e) {
	    throw new Exception("Failed to parse hh:mm:ss pattern");
	}
	
	if ((mm > 59) || (ss > 59))
	    throw new Exception("Failed to parse hh:mm:ss pattern");
	
	long periodLong = 1000 * (ss + 60 * (mm + 60 * hh));
	
	return periodLong;
    }
    
    // ------------------------------------------------------------
    public static String to_hh_mm_ss(long time) {
	
	long ss = time / 1000;
	long hh = ss / 3600;
	ss -= hh * 3600;
	long mm = ss / 60;
	ss -= mm * 60;
	
	return (hh < 10 ? "0" : "") + hh + ":" + (mm < 10 ? "0" : "") + mm
	    + ":" + (ss < 10 ? "0" : "") + ss;
    }
    
    // ------------------------------------------------------------
    public static String to_hh_mm_ss_sss(long time) {
	
	// 		long sss = time / 1000;
	// 		long hh = sss / 3600000;
	long sss = time;
	long hh = sss / 3600000;
	sss -= hh * 3600000;
	long mm = sss / 60000;
	sss -= mm * 60000;
	long ss = sss / 1000;
	sss -= ss * 1000;
	
	return (hh < 10 ? "0" : "") + hh + ":" + (mm < 10 ? "0" : "") + mm
	    + ":" + (ss < 10 ? "0" : "") + ss
	    + "." + (sss < 10 ? "00" : (sss < 100 ? "0" : "")) + sss;
    }
    
    // // ------------------------------------------------------------
    // public static List<Set<String>> toListOfSet(String[] array) {
    
    // 	List<Set<String>> result = new LinkedList<Set<String>>();
    // 	for (int i = 0; i < array.length; i++)
    // 	    result.add(toSet(array[i].split(",")));
    // 	return result;
    // }
    
    // ------------------------------------------------------------
    public static Set<String> toSet(String[] array) {
	
	Set<String> result = new HashSet<String>();
	for (int i = 0; i < array.length; i++)
	    result.add(array[i]);
	return result;
    }
    
    // // ------------------------------------------------------------
    // public static String toString(List<Set<String>> values, 
    // 				  String separator) {
    
    // 	if (values == null)
    // 	    return "";
    
    // 	String result = "";
    
    // 	boolean firstItem = true;
    // 	for (Set<String> key : values)
    // 	    if (firstItem) {
    // 		result = toString(key, ",");
    // 		firstItem = false;
    // 	    }
    // 	    else
    // 		result += separator + toString(key, ",");
    // 	return result;
    // }
    
    // // ------------------------------------------------------------
    // public static String toString(List<String> values, String separator) {
    
    // 	if (values == null)
    // 	    return "";
    
    // 	String result = "";
    
    // 	boolean firstItem = true;
    // 	for (String key : values)
    // 	    if (firstItem) {
    // 		result = key;
    // 		firstItem = false;
    // 	    }
    // 	    else
    // 		result += separator + key;
    
    // 	return result;
    // }
    
    // ------------------------------------------------------------
    public static String toString(BitSet values) {
	
	if (values == null)
	    return "";
	
	String result = "";
	
	for (int i=0; i < values.length(); i++)
	    if (values.get(i))
		result += "1";
	    else
		result += "0";
	
	return result;
    }
    
    // ------------------------------------------------------------
    public static BitSet toBitSet(String str) {
	
	if (str == null)
	    return null;
	
	BitSet result = new BitSet(str.length());
	for (int i=0; i < str.length(); i++)
	    if (str.charAt(i) == '1')
		result.set(i);
	
	return result;
    }
    
    // ------------------------------------------------------------
    public static String toString(Collection<String> values,
				  String separator) {
	
	if (values == null)
	    return "";
	
	String result = "";
	
	boolean firstItem = true;
	for (String key: values)
	    if (firstItem) {
		result = key;
		firstItem = false;
	    }
	    else
		result += separator + key;
	
	return result;
    }
    
      // ------------------------------------------------------------
    public static String toString(Map<String,String> map,
				  String separator) {
	
	if (map == null)
	    return "";
	
	String result = "";
	boolean first = true;
	for (String name: map.keySet()) {
	    if (first) {
		first = false;
            }
            else {
		result += separator;
            }
	    String value = map.get(name);
	    result += name + "=\"" + value + "\"";
	}

	return result;
    }
    
    // ------------------------------------------------------------
    public static String toString(Catalog catalog,
				  String separator) {
	
	if (catalog == null)
	    return "";
	
	String result;
	
	boolean firstItem = true;
	result = "";
	for (String key: catalog.keySet()) {
	    BitSet bitset = catalog.getFragments(key);
	    if (! firstItem) result += separator;
	    else firstItem = false;
	    
	    result += key + "[" + toString(bitset) + "]";
	}
	return result;
    }
    
    // ------------------------------------------------------------
    public static String toString(String[] values, String separator) {
	
    	if (values == null)
    	    return "";
	
    	String result = "";
	
    	boolean firstItem = true;
    	for (int i = 0; i < values.length; i++)
    	    if (firstItem) {
    		result = values[i];
    		firstItem = false;
    	    }
    	    else
    		result += separator + values[i];
	
    	return result;
    }
    
    // ------------------------------------------------------------
    public static Set<String> splitToSet(String str, String separator) {
	
	if (str == null) 
	    return null;
	
	String values[];
	if (str.equals("")) 
	    values = new String[0];
	else
	    values = str.split(separator);
	return toSet(values);
    }
    
    // ------------------------------------------------------------
    public static Catalog splitToCatalog(String str, String separator) {
	
	Catalog result = new Catalog();
	
	if (str == null) 
	    return null;
	
	if (str.equals("")) 
	    return result;
	
	// str is of the form xxxx[010101],yyyy[11],zzzz[000010]...
	String values[] = str.split(separator);
	
	for (int i=0; i < values.length; i++) {
	    String str1 = values[i];
	    int c1 = str1.lastIndexOf("[");
	    int c2 = str1.lastIndexOf("]");
	    String key = str1.substring(0, c1);
	    String str2 = str1.substring(c1+1, c2);
	    BitSet bitset = toBitSet(str2);
	    result.put(key, bitset);
	}
	
	return result;
    }
    
    // ------------------------------------------------------------
    public static List<String> splitToList(String str, String separator) {
	
	if (str == null)
	    return null;
	
	String[] values = str.split(separator);
	return toList(values);
    }
    
    // ------------------------------------------------------------
    public static Set<String> toSet(Collection<String> values) {
	
	if (values == null)
	    return null;
	
	return new HashSet<String>(values);
    }
    
    // ------------------------------------------------------------
    public static List<String> toList(Collection<String> values) {
	
	if (values == null)
	    return null;
	
	return new LinkedList<String>(values);
    }
    
    // ------------------------------------------------------------
    public static List<String> toList(String[] values) {
	
	if (values == null)
	    return null;
	
	List<String> result = new LinkedList<String>();
	for (String v: values)
	    result.add(v);
	return result;
    }
    
    
    
    // ------------------------------------------------------------
    /**
     * Trim an identifier and suppress from it unsuitable characters.  
     * @param id the identifier
     * @return The identifier with only valid characters, or null if the id is null
     */
    public static String cleanIdentifier(String id) {	
	
    	if (id == null) return null;
    	
    	char[] valid = new char[id.length()];
    	int i=0;  // count of chars in source
    	int j=0;  // count of chars in valid
	
    	while (i < id.length()) {
	    int cp = id.codePointAt(i);
	    if (isValidForIdentifier(cp)) { 
		valid[j]=(char)cp;
		j++;
	    }
	    i++;
    	}
    	return String.copyValueOf(valid, 0, j).trim();
    }
    
    // ------------------------------------------------------------
    /**
     *  Checks that a string contains only characters suitable for use
     *  as an identifier
     *  @param id the string to check
     *  @return null if id is null or contains only suitable
     *  characters, and the non-empty string containing the list of
     *  invalid characters otherwise
     */
    static public String checkIdentifier(String id) {
	
    	if (id == null) return null;
	
    	char[] invalid = new char[id.length()];
    	int i=0;  // count of chars in source
    	int j=0;  // count of chars in invalid
	
    	while (i<id.length()) {
	    int cp = id.codePointAt(i);
	    if (! isValidForIdentifier(cp)) { 
		invalid[j]=(char)cp;
		j++;
	    }
	    i++;
    	}
    	if (j==0) return null;
    	else return String.copyValueOf(invalid, 0, j); 	
    }
    
    
    
    // ------------------------------------------------------------
    /**
     * Tests if a character is suitable for use in an identifier.
     * Only iso-latin-1 characters are allowed: ASCII digits, letters
     * (including national letters) Only iso-latin-1 characters are
     * allowed: ASCII digits, letters (including national letters)
     * space, and ascii symbols often used in email addresses: full
     * stop, at, underscore, dash
     * 
     * @param cp the code point of the character
     * @return true if the character is valid, false otherwise.
     */
    static public boolean isValidForIdentifier(int cp) {
    	return 
	    ((cp>=0x0061) && (cp<=0x007A)) ||   // small letters
	    ((cp>=0x0040) && (cp<=0x005A)) ||   // '@' + capital letters
	    ((cp>=0x0030) && (cp<=0x0039)) ||   // digits
	    (cp==0x002D)                   ||   // dash
	    (cp==0x002E)                   ||   // full stop
	    (cp==0x005F)                   ||   // underscore
	    (cp==0x0020)                   ||   // space
	    ((cp>=0x00C0) && (cp<=0x00D6)) ||   // Right half of the
						// supplement latin 1
						// block (i.e extra
						// letters)
	    ((cp>=0x00D8) && (cp<=0x00F6)) ||   //  minus mathematical 
	    ((cp>=0x00F8) && (cp<=0x00FF));     //  operators
	
    }
    
    
    // ------------------------------------------------------------
    static public long unsignedHexToLong(String value) {
	if (value.charAt(0) > '7')
	    return new BigInteger(value, 16).longValue();
	else 
	    return Long.parseLong(value, 16);
    }
    
    // ------------------------------------------------------------
    public static void main(String[] args) {
	
	BitSet bs1 = new BitSet(30);
	bs1.set(1); bs1.set(3); bs1.set(5); bs1.set(7); bs1.set(9);
	BitSet bs2 = new BitSet(30);
	bs2.set(11); bs2.set(13); bs2.set(17); bs2.set(19); bs2.set(23);
	String str1 = DataFormatting.toString(bs1);
	String str2 = DataFormatting.toString(bs2);
	System.out.println(str1 + " / " + str2);
	System.out.println(DataFormatting.toBitSet(str1) 
			   + " / "
			   + DataFormatting.toBitSet(str2));
	
	Catalog catalog = new Catalog();
	catalog.put("bs1", bs1);
	catalog.put("bs2", bs2);
	String str3 = DataFormatting.toString(catalog, ",");
	System.out.println(str3);
	catalog = splitToCatalog(str3, ",");
	System.out.println(catalog);
	System.exit(0);
    }
}
