/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;

import java.util.Set;
import java.util.HashSet;
import java.util.Map;
import java.util.HashMap;
import java.util.BitSet;
import java.util.concurrent.atomic.AtomicBoolean;

import casa.dodwan.docware.Descriptor;

public class Host {
    
    public String id;

    // public String presenceId;

    // Identifier of the last profile registered for that host
    public int pid = -1;

    // Map of the messages (that match our interest profile) this
    // peerbor host can send us. For each such message, the bitset
    // indicates which payload fragments are available in the
    // peer's cache.
    public Catalog canProvide = new Catalog();

    // Identifiers of duplicate messages that are available in both
    // this host and the local host
    public Catalog duplicates = new Catalog();

    // Map of the messages we could send to that host (because they
    // match its interest profile and it apparently does not own
    // them). For each such message, the bitset indicates which
    // payload fragments are available in the local cache.
    public Catalog couldUse = new Catalog();

    // Identifier of the message is currently expected from that host
    public String expected = null;

    // Interest profile of that host
    public Set<Descriptor> profile =
	new HashSet<Descriptor>();

    public long lastAnnounce = 0;

    public int helloCnt = 0;
    
    public int lastAckedPid = 0;

    public AtomicBoolean busy = new AtomicBoolean(false);

    // ------------------------------------------------------------
    public Host(String id) {
	this.id = id;
	long now = System.currentTimeMillis();
	lastAnnounce = now;
    }
}
