/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;

public class Pattern {
	public static void main(String[] args) {
		try {
			test();
			System.out.println("SUCCESS");
		} catch (Exception e) {
			System.out.println(e);
		}
	}

	/**
	 * Return whether a string matches a pattern defined like filenames patterns
	 * in a shell ('?' matches any character, '*' matches many characters).
	 * 
	 * @param pattern
	 *            the pattern
	 * @param string
	 *            the string
	 * @return true if the string matches the pattern
	 */
	public static boolean matches(String pattern, String string) {
		return regexMatches(pattern, string);
	}

	private static boolean regexMatches(String pattern, String string) {
		StringBuffer buffer = new StringBuffer(pattern);
		StringBuffer bufferRegexp = new StringBuffer();
		bufferRegexp.append('^');
		for (int i = 0; i < buffer.length(); i++) {
			char c = buffer.charAt(i);
			switch (c) {
			case '*':
				bufferRegexp.append(".*");
				break;
			case '?':
				bufferRegexp.append(".");
				break;
			case '.':
			case '{':
			case '}':
			case '|':
			case '(':
			case ')':
			case '\\':
				bufferRegexp.append("\\" + c);
				break;
			default:
				bufferRegexp.append(buffer.charAt(i));
				break;
			}
		}
		bufferRegexp.append('$');
		String regexp = new String(bufferRegexp);
		return java.util.regex.Pattern.matches(regexp, string);
	}

	private static void test() throws Exception {

		// let's start with a few cases that should often happens...
		test("abc", "abc", true);
		test("abc", "123", false);
		test("*", "abc", true);
		test("a*", "abc", true);
		test("*b*", "abc", true);
		test("*b*", "123", false);
		test("*b*", "ab", true);
		test("*a*", "ab", true);
		test("*c", "abc", true);
		test("*d", "abc", false);
		test("a*b", "ab", true);
		// is 'counting' well done ?
		test("??c", "abc", true);
		test("??c", "123", false);
		test("???", "123", true);
		test("???", "1234", false);
		test("????", "123", false);
		test("??*", "1", false);
		test("??*", "12", true);
		test("??*", "123", true);
		test("?*?", "1", false);
		test("?*?", "12", true);
		test("?*?", "123", true);
		test("*??", "1", false);
		test("*??", "12", true);
		test("*??", "123", true);
		// there could be some specials chars that bugs...
		char[] specials = { '.', '/', /* '\\', */'|', '(', ')', '{', '}' };
		for (int i = 0; i < specials.length; i++) {
			test("*", "toto" + specials[i] + "titi", true);
			test("*" + specials[i] + "*", "toto" + specials[i] + "titi", true);
			test("*" + specials[i] + "*" + specials[i] + "*", "toto"
					+ specials[i] + "titi", false);
		}

	}

	private static void test(String pattern, String s, boolean shouldBe)
			throws Exception {

		boolean b = matches(pattern, s);
		if (b != shouldBe) {
			throw new Exception("Test failed: " + pattern + ", " + s
					+ ": was '" + b + "' (should be '" + shouldBe + "')");
		}
	}
}
