/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/


package casa.dodwan.util;

import java.io.IOException;
import java.net.ServerSocket;

import casa.dodwan.config.DodwanEnvironment;


/**
 * A simple lock mechanism. 
   The method used is to open a server socket on a specific port.
 */

public class SimpleLock {

    private static ServerSocket serverSocket_ = null;


    // --------------------------------------------------------------------------
    /**
     * Acquires the lock if possible
     * @return true if the lock could be acquired, false if it is already hold.
     */
    public static boolean tryLock() {
	// The lock socket port is the DoDWAN console port - 1;
	int port = DodwanEnvironment.getInstance().console_port - 1;
	try{
	    serverSocket_ = new ServerSocket(port);
	}
	catch (IOException ioe) {
	    return false;
	}
	catch (SecurityException e) {
	    System.err.println("Could not open a socket on port #" + port + " for security reasons");
	    return false;
	}
	return true;
	
   }


    // --------------------------------------------------------------------------
    /**
     * Releases the lock
     */
    public static void unlock() throws Exception{
	serverSocket_.close();
	serverSocket_ = null; 
    }



}

