/* *****************************************************************************
 * Copyright (c) IRISA Laboratory, 
 * Universite Bretagne Sud, France
 * <http://www-casa.irisa.fr/dodwan>
 *
 * This file is part of DoDWAN.
 * 
 * DoDWAN is free software: you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software 
 * Foundation, either version 3 of the License, or any later
 * version.
 * 
 * DoDWAN is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * DoDWAN.  If not, see <http://www.gnu.org/licenses/>.
 * ****************************************************************************/



package casa.dodwan.util;

/**
 * The class <code>Verbosity</code> is meant to help controlling the verbosity
 * of other classes, especially for debugging purposes.
 * 
 * When developping a new class whose verbosity it is desirable to control, just
 * insert a public instance of class <code>Verbosity</code> in this new class:
 * 
 * <code>public Verbosity verbosity = new Verbosity();</code>
 * 
 * Enabling and disabling the verbosity status of the class is achieved by
 * calling methods <code>enable()</code> and <code>disable()</code>
 * respectively. Debugging comments should be conditioned by a test of the
 * verbosity status, as shown below.
 * 
 * <code>if verbosity.isEnabled() {
 *    System.out.println("Debugging comment"); 
 * }</code>
 * 
 */
public class Verbosity {
	protected boolean verbosity_ = false;

	public void disable() {
		verbosity_ = false;
	}

	public void enable() {
		verbosity_ = true;
	}

	public boolean isEnabled() {
		return verbosity_;
	}
}
